{% extends "base.html" %}

{% block title %}About — Django autoreload example{% endblock %}

{% block content %}
<h1>About</h1>
<p>This example demonstrates <strong>asgi-autoreload</strong> with Django and Daphne.</p>
<p>The autoreload script is injected automatically into every HTML response — no changes to templates needed.</p>
{% endblock %}
