<!doctype html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <title>FastAPI build command example</title>
  <link rel="stylesheet" href="/static/bundle.css">
</head>
<body>
  <h1>Hello, world</h1>
  <p>
    Edit <code>src/style.css</code> and watch the stylesheet rebuild and the
    browser reload automatically.
  </p>
</body>
</html>
